{"filter":false,"title":"FoxHound-Dialect-ALASQL.js","tooltip":"/source/dialects/ALASQL/FoxHound-Dialect-ALASQL.js","undoManager":{"mark":18,"position":18,"stack":[[{"start":{"row":0,"column":0},"end":{"row":783,"column":0},"action":"remove","lines":["/**","* FoxHound ALASQL Dialect","*","* @license MIT","*","* For an ALASQL query override:","// An underscore template with the following values:","//      <%= DataElements %> = Field1, Field2, Field3, Field4","//      <%= Begin %>        = 0","//      <%= Cap %>          = 10","//      <%= Filter %>       = WHERE StartDate > :MyStartDate","//      <%= Sort %>         = ORDER BY Field1","// The values are empty strings if they aren't set.","*","* @author Steven Velozo <steven@velozo.com>","* @class FoxHoundDialectALASQL","*/","var libUnderscore = require('underscore');","","var FoxHoundDialectALASQL = function()","{","\t/**","\t * Generate a table name from the scope.","\t * ","\t * Because ALASQL is all in-memory, and can be run in two modes (anonymous","\t * working on arrays or table-based) we are going to make this a programmable","\t * value.  Then we can share the code across both providers.","\t *","\t * @method: generateTableName","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateTableName = function(pParameters)","\t{","\t\treturn ' '+pParameters.scope;","\t};","\t","\t/**","\t * Escape columns, because ALASQL has more reserved KWs than most SQL dialects","\t */","\tvar escapeColumn = (pColumn, pParameters) =>","\t{","\t\tif (pColumn.indexOf('.') < 0)","\t\t{","\t\t\treturn '`'+pColumn+'`';","\t\t}","\t\telse","\t\t{","\t\t\t// This could suck if the scope is not the same","\t\t\tvar tmpTableName = pParameters.scope;","\t\t\tif (pColumn.indexOf(tmpTableName+'.'))","\t\t\t{","\t\t\t\treturn '`'+pColumn.replace(tmpTableName+'.', '')+'`';","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\t// This doesn't work well but we'll try it.","\t\t\t\treturn '`'+pColumn+'`';","\t\t\t}","\t\t}","\t};","","\t/**","\t * Generate a field list from the array of dataElements","\t *","\t * Each entry in the dataElements is a simple string","\t *","\t * @method: generateFieldList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateFieldList = function(pParameters)","\t{","\t\tvar tmpDataElements = pParameters.dataElements;","\t\tif (!Array.isArray(tmpDataElements) || tmpDataElements.length < 1)","\t\t{","\t\t\treturn ' *';","\t\t}","","\t\tvar tmpFieldList = ' ';","\t\tfor (var i = 0; i < tmpDataElements.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpFieldList += ', ';","\t\t\t}","\t\t\ttmpFieldList += escapeColumn(tmpDataElements[i], pParameters);","\t\t}","\t\treturn tmpFieldList;","\t};","","\t/**","\t * Generate a query from the array of where clauses","\t *","\t * Each clause is an object like:","\t\t{","\t\t\tColumn:'Name', ","\t\t\tOperator:'EQ', ","\t\t\tValue:'John', ","\t\t\tConnector:'And', ","\t\t\tParameter:'Name'","\t\t}","\t *","\t * @method: generateWhere","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the WHERE clause prefixed with WHERE, or an empty string if unnecessary","\t */","\tvar generateWhere = function(pParameters)","\t{","\t\tvar tmpFilter = Array.isArray(pParameters.filter) ? pParameters.filter : [];","\t\tvar tmpTableName = generateTableName(pParameters).trim();","","\t\tif (!pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t// Check if there is a Deleted column on the Schema. If so, we add this to the filters automatically (if not already present)","\t\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\tvar tmpSchemaEntry = tmpSchema[i];","","\t\t\t\tif (tmpSchemaEntry.Type === 'Deleted')","\t\t\t\t{","\t\t\t\t\tvar tmpHasDeletedParameter = false;","","\t\t\t\t\t//first, check to see if filters are already looking for Deleted column","\t\t\t\t\tif (tmpFilter.length > 0)","\t\t\t\t\t{","\t\t\t\t\t\tfor (var x = 0; x < tmpFilter.length; x++)","\t\t\t\t\t\t{","\t\t\t\t\t\t\tif (tmpFilter[x].Column === tmpSchemaEntry.Column)","\t\t\t\t\t\t\t{","\t\t\t\t\t\t\t\ttmpHasDeletedParameter = true;","\t\t\t\t\t\t\t\tbreak;","\t\t\t\t\t\t\t}","\t\t\t\t\t\t}","\t\t\t\t\t}","\t\t\t\t\tif (!tmpHasDeletedParameter)","\t\t\t\t\t{","\t\t\t\t\t\t//if not, we need to add it","\t\t\t\t\t\ttmpFilter.push(","\t\t\t\t\t\t{","\t\t\t\t\t\t\tColumn: tmpTableName + '.' + tmpSchemaEntry.Column,","\t\t\t\t\t\t\tOperator: '=',","\t\t\t\t\t\t\tValue: 0,","\t\t\t\t\t\t\tConnector: 'AND',","\t\t\t\t\t\t\tParameter: 'Deleted'","\t\t\t\t\t\t});","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t}","","\t\tif (tmpFilter.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpWhere = ' WHERE';","","\t\t// This is used to disable the connectors for subsequent queries.","\t\t// Only the open parenthesis operator uses this, currently.","\t\tvar tmpLastOperatorNoConnector = false;","","\t\tfor (var i = 0; i < tmpFilter.length; i++)","\t\t{","\t\t\tif ((tmpFilter[i].Connector != 'NONE') && (tmpFilter[i].Operator != ')') && (tmpWhere != ' WHERE') && (tmpLastOperatorNoConnector == false))","\t\t\t{","\t\t\t\ttmpWhere += ' '+tmpFilter[i].Connector;","\t\t\t}","","\t\t\ttmpLastOperatorNoConnector = false;","","\t\t\tvar tmpColumnParameter;","","\t\t\tif (tmpFilter[i].Operator === '(')","\t\t\t{","\t\t\t\t// Open a logical grouping","\t\t\t\ttmpWhere += ' (';","\t\t\t\ttmpLastOperatorNoConnector = true;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === ')')","\t\t\t{","\t\t\t\t// Close a logical grouping","\t\t\t\ttmpWhere += ' )';","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IN')","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' ( :'+tmpColumnParameter+' )';","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IS NOT NULL')","\t\t\t{","\t\t\t\t// IS NOT NULL is a special operator which doesn't require a value, or parameter","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator;","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' :'+tmpColumnParameter;","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t}","","\t\treturn tmpWhere;","\t};","","\t/**","\t * Generate an ORDER BY clause from the sort array","\t *","\t * Each entry in the sort is an object like:","\t * {Column:'Color',Direction:'Descending'}","\t *","\t * @method: generateOrderBy","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateOrderBy = function(pParameters)","\t{","\t\tvar tmpOrderBy = pParameters.sort;","\t\tif (!Array.isArray(tmpOrderBy) || tmpOrderBy.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpOrderClause = ' ORDER BY';","\t\tfor (var i = 0; i < tmpOrderBy.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpOrderClause += ',';","\t\t\t}","\t\t\ttmpOrderClause += ' '+escapeColumn(tmpOrderBy[i].Column, pParameters);","","\t\t\tif (tmpOrderBy[i].Direction == 'Descending')","\t\t\t{","\t\t\t\ttmpOrderClause += ' DESC';","\t\t\t}","\t\t}","\t\treturn tmpOrderClause;","\t};","","\t/**","\t * Generate the limit clause","\t *","\t * @method: generateLimit","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateLimit = function(pParameters)","\t{","\t\tif (!pParameters.cap)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpLimit = ' LIMIT';","\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}","\t\t// Cap is required for a limit clause.","\t\ttmpLimit += ' ' + pParameters.cap;","","\t\treturn tmpLimit;","\t};","","\t/**","\t * Generate the update SET clause","\t *","\t * @method: generateUpdateSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateSetters = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpUpdate = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (pParameters.query.disableAutoDateStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateDate')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (pParameters.query.disableAutoUserStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateIDUser')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteDate':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// These are all ignored on update","\t\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tvar tmpColumnDefaultParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnDefaultParameter;","","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnDefaultParameter] = tmpRecords[0][tmpColumn];","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the update-delete SET clause","\t *","\t * @method: generateUpdateDeleteSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateDeleteSetters = function(pParameters)","\t{","\t\tif (pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t//Don't generate an UPDATE query if Delete tracking is disabled","\t\t\treturn false;","\t\t}","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCurrentColumn = 0;","\t\tvar tmpHasDeletedField = false;","\t\tvar tmpUpdate = '';","\t\t// No hash table yet, so, we will just linear search it for now.","\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\tvar tmpSchemaEntry = {Type:'Default'};","\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t{","\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\ttmpSchemaEntry = tmpSchema[i];","","\t\t\tvar tmpUpdateSql = null;","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'Deleted':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = 1';","\t\t\t\t\ttmpHasDeletedField = true; //this field is required in order for query to be built","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// Delete operation is an Update, so we should stamp the update time","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpSchemaEntry.Column+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\t//DON'T allow update of other fields in this query","\t\t\t\t\tcontinue;","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","","\t\t\ttmpUpdate += tmpUpdateSql;","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (!tmpHasDeletedField ||","\t\t\ttmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetValues = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpCreateSet += ',';","\t\t\t}","","\t\t\t//define a re-usable method for setting up field definitions in a default pattern","\t\t\tvar buildDefaultDefinition = function()","\t\t\t{","\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t// Set the query parameter","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpRecords[0][tmpColumn];","\t\t\t};","","\t\t\tvar tmpColumnParameter;","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NULL';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'AutoGUID':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse if (tmpRecords[0][tmpColumn] &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn].length >= 5 &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn] !== '0x0000000000000000') //stricture default","\t\t\t\t\t{","\t\t\t\t\t\t// Allow consumer to override AutoGUID","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.UUID;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\tif (pParameters.query.disableAutoDateStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NOW()';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\tif (pParameters.query.disableAutoUserStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use an appended number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpCreateSet === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpCreateSet;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetList = function(pParameters)","\t{","\t\t// The records were already validated by generateCreateSetValues","\t\tvar tmpRecords = pParameters.query.records;","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tdefault:","\t\t\t\t\tif (tmpCreateSet != '')","\t\t\t\t\t{","\t\t\t\t\t\ttmpCreateSet += ',';","\t\t\t\t\t}","\t\t\t\t\ttmpCreateSet += ' '+escapeColumn(tmpColumn, pParameters);","\t\t\t\t\tbreak;","\t\t\t}","\t\t}","","\t\treturn tmpCreateSet;","\t};","","","\tvar Create = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpCreateSetList = generateCreateSetList(pParameters);","\t\tvar tmpCreateSetValues = generateCreateSetValues(pParameters);","","\t\tif (!tmpCreateSetValues)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'INSERT INTO'+tmpTableName+' ('+tmpCreateSetList+') VALUES ('+tmpCreateSetValues+');';","\t};","","","\t/**","\t* Read one or many records","\t*","\t* Some examples:","\t* SELECT * FROM WIDGETS;","\t* SELECT * FROM WIDGETS LIMIT 0, 20;","\t* SELECT * FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20 WHERE LastName = 'Smith';","\t*","\t* @method Read","\t* @param {Object} pParameters SQL Query parameters","\t* @return {String} Returns the current Query for chaining.","\t*/","\tvar Read = function(pParameters)","\t{","\t\tvar tmpFieldList = generateFieldList(pParameters);","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpOrderBy = generateOrderBy(pParameters);","\t\tvar tmpLimit = generateLimit(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:tmpFieldList, TableName:tmpTableName, Where:tmpWhere, OrderBy:tmpOrderBy, Limit:tmpLimit});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Read Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT'+tmpFieldList+' FROM'+tmpTableName+tmpWhere+tmpOrderBy+tmpLimit+';';","\t};","","\tvar Update = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateSetters = generateUpdateSetters(pParameters);","","\t\tif (!tmpUpdateSetters)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateSetters+tmpWhere+';';","\t};","","\tvar Delete = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateDeleteSetters = generateUpdateDeleteSetters(pParameters);","","\t\tif (tmpUpdateDeleteSetters)","\t\t{","\t\t\t//If it has a deleted bit, update it instead of actually deleting the record","\t\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateDeleteSetters+tmpWhere+';';","\t\t}","\t\telse","\t\t{","\t\t\treturn 'DELETE FROM'+tmpTableName+tmpWhere+';';","\t\t}","\t};","","\tvar Count = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:[], TableName:tmpTableName, Where:tmpWhere, OrderBy:'', Limit:''});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Count Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT COUNT(*) AS RowCount FROM'+tmpTableName+tmpWhere+';';","\t};","","\tvar tmpDialect = ({","\t\tCreate: Create,","\t\tRead: Read,","\t\tUpdate: Update,","\t\tDelete: Delete,","\t\tCount: Count","\t});","","\t/**","\t * Dialect Name","\t *","\t * @property name","\t * @type string","\t */","\tObject.defineProperty(tmpDialect, 'name',","\t\t{","\t\t\tget: function() { return 'ALASQL'; },","\t\t\tenumerable: true","\t\t});","","\treturn tmpDialect;","};","","module.exports = new FoxHoundDialectALASQL();",""],"id":988},{"start":{"row":0,"column":0},"end":{"row":783,"column":0},"action":"insert","lines":["/**","* FoxHound ALASQL Dialect","*","* @license MIT","*","* For an ALASQL query override:","// An underscore template with the following values:","//      <%= DataElements %> = Field1, Field2, Field3, Field4","//      <%= Begin %>        = 0","//      <%= Cap %>          = 10","//      <%= Filter %>       = WHERE StartDate > :MyStartDate","//      <%= Sort %>         = ORDER BY Field1","// The values are empty strings if they aren't set.","*","* @author Steven Velozo <steven@velozo.com>","* @class FoxHoundDialectALASQL","*/","var libUnderscore = require('underscore');","","var FoxHoundDialectALASQL = function()","{","\t/**","\t * Generate a table name from the scope.","\t * ","\t * Because ALASQL is all in-memory, and can be run in two modes (anonymous","\t * working on arrays or table-based) we are going to make this a programmable","\t * value.  Then we can share the code across both providers.","\t *","\t * @method: generateTableName","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateTableName = function(pParameters)","\t{","\t\treturn ' '+pParameters.scope;","\t};","\t","\t/**","\t * Escape columns, because ALASQL has more reserved KWs than most SQL dialects","\t */","\tvar escapeColumn = (pColumn, pParameters) =>","\t{","\t\tif (pColumn.indexOf('.') < 0)","\t\t{","\t\t\treturn '`'+pColumn+'`';","\t\t}","\t\telse","\t\t{","\t\t\t// This could suck if the scope is not the same","\t\t\tvar tmpTableName = pParameters.scope;","\t\t\tif (pColumn.indexOf(tmpTableName+'.') > -1)","\t\t\t{","\t\t\t\treturn '`'+pColumn.replace(tmpTableName+'.', '')+'`';","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\t// This doesn't work well but we'll try it.","\t\t\t\treturn '`'+pColumn+'`';","\t\t\t}","\t\t}","\t};","","\t/**","\t * Generate a field list from the array of dataElements","\t *","\t * Each entry in the dataElements is a simple string","\t *","\t * @method: generateFieldList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateFieldList = function(pParameters)","\t{","\t\tvar tmpDataElements = pParameters.dataElements;","\t\tif (!Array.isArray(tmpDataElements) || tmpDataElements.length < 1)","\t\t{","\t\t\treturn ' *';","\t\t}","","\t\tvar tmpFieldList = ' ';","\t\tfor (var i = 0; i < tmpDataElements.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpFieldList += ', ';","\t\t\t}","\t\t\ttmpFieldList += escapeColumn(tmpDataElements[i], pParameters);","\t\t}","\t\treturn tmpFieldList;","\t};","","\t/**","\t * Generate a query from the array of where clauses","\t *","\t * Each clause is an object like:","\t\t{","\t\t\tColumn:'Name', ","\t\t\tOperator:'EQ', ","\t\t\tValue:'John', ","\t\t\tConnector:'And', ","\t\t\tParameter:'Name'","\t\t}","\t *","\t * @method: generateWhere","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the WHERE clause prefixed with WHERE, or an empty string if unnecessary","\t */","\tvar generateWhere = function(pParameters)","\t{","\t\tvar tmpFilter = Array.isArray(pParameters.filter) ? pParameters.filter : [];","\t\tvar tmpTableName = generateTableName(pParameters).trim();","","\t\tif (!pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t// Check if there is a Deleted column on the Schema. If so, we add this to the filters automatically (if not already present)","\t\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\tvar tmpSchemaEntry = tmpSchema[i];","","\t\t\t\tif (tmpSchemaEntry.Type === 'Deleted')","\t\t\t\t{","\t\t\t\t\tvar tmpHasDeletedParameter = false;","","\t\t\t\t\t//first, check to see if filters are already looking for Deleted column","\t\t\t\t\tif (tmpFilter.length > 0)","\t\t\t\t\t{","\t\t\t\t\t\tfor (var x = 0; x < tmpFilter.length; x++)","\t\t\t\t\t\t{","\t\t\t\t\t\t\tif (tmpFilter[x].Column === tmpSchemaEntry.Column)","\t\t\t\t\t\t\t{","\t\t\t\t\t\t\t\ttmpHasDeletedParameter = true;","\t\t\t\t\t\t\t\tbreak;","\t\t\t\t\t\t\t}","\t\t\t\t\t\t}","\t\t\t\t\t}","\t\t\t\t\tif (!tmpHasDeletedParameter)","\t\t\t\t\t{","\t\t\t\t\t\t//if not, we need to add it","\t\t\t\t\t\ttmpFilter.push(","\t\t\t\t\t\t{","\t\t\t\t\t\t\tColumn: tmpTableName + '.' + tmpSchemaEntry.Column,","\t\t\t\t\t\t\tOperator: '=',","\t\t\t\t\t\t\tValue: 0,","\t\t\t\t\t\t\tConnector: 'AND',","\t\t\t\t\t\t\tParameter: 'Deleted'","\t\t\t\t\t\t});","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t}","","\t\tif (tmpFilter.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpWhere = ' WHERE';","","\t\t// This is used to disable the connectors for subsequent queries.","\t\t// Only the open parenthesis operator uses this, currently.","\t\tvar tmpLastOperatorNoConnector = false;","","\t\tfor (var i = 0; i < tmpFilter.length; i++)","\t\t{","\t\t\tif ((tmpFilter[i].Connector != 'NONE') && (tmpFilter[i].Operator != ')') && (tmpWhere != ' WHERE') && (tmpLastOperatorNoConnector == false))","\t\t\t{","\t\t\t\ttmpWhere += ' '+tmpFilter[i].Connector;","\t\t\t}","","\t\t\ttmpLastOperatorNoConnector = false;","","\t\t\tvar tmpColumnParameter;","","\t\t\tif (tmpFilter[i].Operator === '(')","\t\t\t{","\t\t\t\t// Open a logical grouping","\t\t\t\ttmpWhere += ' (';","\t\t\t\ttmpLastOperatorNoConnector = true;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === ')')","\t\t\t{","\t\t\t\t// Close a logical grouping","\t\t\t\ttmpWhere += ' )';","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IN')","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' ( :'+tmpColumnParameter+' )';","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IS NOT NULL')","\t\t\t{","\t\t\t\t// IS NOT NULL is a special operator which doesn't require a value, or parameter","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator;","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' :'+tmpColumnParameter;","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t}","","\t\treturn tmpWhere;","\t};","","\t/**","\t * Generate an ORDER BY clause from the sort array","\t *","\t * Each entry in the sort is an object like:","\t * {Column:'Color',Direction:'Descending'}","\t *","\t * @method: generateOrderBy","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateOrderBy = function(pParameters)","\t{","\t\tvar tmpOrderBy = pParameters.sort;","\t\tif (!Array.isArray(tmpOrderBy) || tmpOrderBy.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpOrderClause = ' ORDER BY';","\t\tfor (var i = 0; i < tmpOrderBy.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpOrderClause += ',';","\t\t\t}","\t\t\ttmpOrderClause += ' '+escapeColumn(tmpOrderBy[i].Column, pParameters);","","\t\t\tif (tmpOrderBy[i].Direction == 'Descending')","\t\t\t{","\t\t\t\ttmpOrderClause += ' DESC';","\t\t\t}","\t\t}","\t\treturn tmpOrderClause;","\t};","","\t/**","\t * Generate the limit clause","\t *","\t * @method: generateLimit","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateLimit = function(pParameters)","\t{","\t\tif (!pParameters.cap)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpLimit = ' LIMIT';","\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}","\t\t// Cap is required for a limit clause.","\t\ttmpLimit += ' ' + pParameters.cap;","","\t\treturn tmpLimit;","\t};","","\t/**","\t * Generate the update SET clause","\t *","\t * @method: generateUpdateSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateSetters = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpUpdate = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (pParameters.query.disableAutoDateStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateDate')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (pParameters.query.disableAutoUserStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateIDUser')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteDate':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// These are all ignored on update","\t\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tvar tmpColumnDefaultParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnDefaultParameter;","","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnDefaultParameter] = tmpRecords[0][tmpColumn];","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the update-delete SET clause","\t *","\t * @method: generateUpdateDeleteSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateDeleteSetters = function(pParameters)","\t{","\t\tif (pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t//Don't generate an UPDATE query if Delete tracking is disabled","\t\t\treturn false;","\t\t}","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCurrentColumn = 0;","\t\tvar tmpHasDeletedField = false;","\t\tvar tmpUpdate = '';","\t\t// No hash table yet, so, we will just linear search it for now.","\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\tvar tmpSchemaEntry = {Type:'Default'};","\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t{","\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\ttmpSchemaEntry = tmpSchema[i];","","\t\t\tvar tmpUpdateSql = null;","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'Deleted':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = 1';","\t\t\t\t\ttmpHasDeletedField = true; //this field is required in order for query to be built","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// Delete operation is an Update, so we should stamp the update time","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpSchemaEntry.Column+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\t//DON'T allow update of other fields in this query","\t\t\t\t\tcontinue;","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","","\t\t\ttmpUpdate += tmpUpdateSql;","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (!tmpHasDeletedField ||","\t\t\ttmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetValues = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpCreateSet += ',';","\t\t\t}","","\t\t\t//define a re-usable method for setting up field definitions in a default pattern","\t\t\tvar buildDefaultDefinition = function()","\t\t\t{","\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t// Set the query parameter","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpRecords[0][tmpColumn];","\t\t\t};","","\t\t\tvar tmpColumnParameter;","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NULL';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'AutoGUID':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse if (tmpRecords[0][tmpColumn] &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn].length >= 5 &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn] !== '0x0000000000000000') //stricture default","\t\t\t\t\t{","\t\t\t\t\t\t// Allow consumer to override AutoGUID","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.UUID;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\tif (pParameters.query.disableAutoDateStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NOW()';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\tif (pParameters.query.disableAutoUserStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use an appended number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpCreateSet === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpCreateSet;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetList = function(pParameters)","\t{","\t\t// The records were already validated by generateCreateSetValues","\t\tvar tmpRecords = pParameters.query.records;","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tdefault:","\t\t\t\t\tif (tmpCreateSet != '')","\t\t\t\t\t{","\t\t\t\t\t\ttmpCreateSet += ',';","\t\t\t\t\t}","\t\t\t\t\ttmpCreateSet += ' '+escapeColumn(tmpColumn, pParameters);","\t\t\t\t\tbreak;","\t\t\t}","\t\t}","","\t\treturn tmpCreateSet;","\t};","","","\tvar Create = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpCreateSetList = generateCreateSetList(pParameters);","\t\tvar tmpCreateSetValues = generateCreateSetValues(pParameters);","","\t\tif (!tmpCreateSetValues)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'INSERT INTO'+tmpTableName+' ('+tmpCreateSetList+') VALUES ('+tmpCreateSetValues+');';","\t};","","","\t/**","\t* Read one or many records","\t*","\t* Some examples:","\t* SELECT * FROM WIDGETS;","\t* SELECT * FROM WIDGETS LIMIT 0, 20;","\t* SELECT * FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20 WHERE LastName = 'Smith';","\t*","\t* @method Read","\t* @param {Object} pParameters SQL Query parameters","\t* @return {String} Returns the current Query for chaining.","\t*/","\tvar Read = function(pParameters)","\t{","\t\tvar tmpFieldList = generateFieldList(pParameters);","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpOrderBy = generateOrderBy(pParameters);","\t\tvar tmpLimit = generateLimit(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:tmpFieldList, TableName:tmpTableName, Where:tmpWhere, OrderBy:tmpOrderBy, Limit:tmpLimit});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Read Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT'+tmpFieldList+' FROM'+tmpTableName+tmpWhere+tmpOrderBy+tmpLimit+';';","\t};","","\tvar Update = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateSetters = generateUpdateSetters(pParameters);","","\t\tif (!tmpUpdateSetters)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateSetters+tmpWhere+';';","\t};","","\tvar Delete = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateDeleteSetters = generateUpdateDeleteSetters(pParameters);","","\t\tif (tmpUpdateDeleteSetters)","\t\t{","\t\t\t//If it has a deleted bit, update it instead of actually deleting the record","\t\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateDeleteSetters+tmpWhere+';';","\t\t}","\t\telse","\t\t{","\t\t\treturn 'DELETE FROM'+tmpTableName+tmpWhere+';';","\t\t}","\t};","","\tvar Count = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:[], TableName:tmpTableName, Where:tmpWhere, OrderBy:'', Limit:''});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Count Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT COUNT(*) AS RowCount FROM'+tmpTableName+tmpWhere+';';","\t};","","\tvar tmpDialect = ({","\t\tCreate: Create,","\t\tRead: Read,","\t\tUpdate: Update,","\t\tDelete: Delete,","\t\tCount: Count","\t});","","\t/**","\t * Dialect Name","\t *","\t * @property name","\t * @type string","\t */","\tObject.defineProperty(tmpDialect, 'name',","\t\t{","\t\t\tget: function() { return 'ALASQL'; },","\t\t\tenumerable: true","\t\t});","","\treturn tmpDialect;","};","","module.exports = new FoxHoundDialectALASQL();",""]}],[{"start":{"row":0,"column":0},"end":{"row":783,"column":0},"action":"remove","lines":["/**","* FoxHound ALASQL Dialect","*","* @license MIT","*","* For an ALASQL query override:","// An underscore template with the following values:","//      <%= DataElements %> = Field1, Field2, Field3, Field4","//      <%= Begin %>        = 0","//      <%= Cap %>          = 10","//      <%= Filter %>       = WHERE StartDate > :MyStartDate","//      <%= Sort %>         = ORDER BY Field1","// The values are empty strings if they aren't set.","*","* @author Steven Velozo <steven@velozo.com>","* @class FoxHoundDialectALASQL","*/","var libUnderscore = require('underscore');","","var FoxHoundDialectALASQL = function()","{","\t/**","\t * Generate a table name from the scope.","\t * ","\t * Because ALASQL is all in-memory, and can be run in two modes (anonymous","\t * working on arrays or table-based) we are going to make this a programmable","\t * value.  Then we can share the code across both providers.","\t *","\t * @method: generateTableName","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateTableName = function(pParameters)","\t{","\t\treturn ' '+pParameters.scope;","\t};","\t","\t/**","\t * Escape columns, because ALASQL has more reserved KWs than most SQL dialects","\t */","\tvar escapeColumn = (pColumn, pParameters) =>","\t{","\t\tif (pColumn.indexOf('.') < 0)","\t\t{","\t\t\treturn '`'+pColumn+'`';","\t\t}","\t\telse","\t\t{","\t\t\t// This could suck if the scope is not the same","\t\t\tvar tmpTableName = pParameters.scope;","\t\t\tif (pColumn.indexOf(tmpTableName+'.') > -1)","\t\t\t{","\t\t\t\treturn '`'+pColumn.replace(tmpTableName+'.', '')+'`';","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\t// This doesn't work well but we'll try it.","\t\t\t\treturn '`'+pColumn+'`';","\t\t\t}","\t\t}","\t};","","\t/**","\t * Generate a field list from the array of dataElements","\t *","\t * Each entry in the dataElements is a simple string","\t *","\t * @method: generateFieldList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateFieldList = function(pParameters)","\t{","\t\tvar tmpDataElements = pParameters.dataElements;","\t\tif (!Array.isArray(tmpDataElements) || tmpDataElements.length < 1)","\t\t{","\t\t\treturn ' *';","\t\t}","","\t\tvar tmpFieldList = ' ';","\t\tfor (var i = 0; i < tmpDataElements.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpFieldList += ', ';","\t\t\t}","\t\t\ttmpFieldList += escapeColumn(tmpDataElements[i], pParameters);","\t\t}","\t\treturn tmpFieldList;","\t};","","\t/**","\t * Generate a query from the array of where clauses","\t *","\t * Each clause is an object like:","\t\t{","\t\t\tColumn:'Name', ","\t\t\tOperator:'EQ', ","\t\t\tValue:'John', ","\t\t\tConnector:'And', ","\t\t\tParameter:'Name'","\t\t}","\t *","\t * @method: generateWhere","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the WHERE clause prefixed with WHERE, or an empty string if unnecessary","\t */","\tvar generateWhere = function(pParameters)","\t{","\t\tvar tmpFilter = Array.isArray(pParameters.filter) ? pParameters.filter : [];","\t\tvar tmpTableName = generateTableName(pParameters).trim();","","\t\tif (!pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t// Check if there is a Deleted column on the Schema. If so, we add this to the filters automatically (if not already present)","\t\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\tvar tmpSchemaEntry = tmpSchema[i];","","\t\t\t\tif (tmpSchemaEntry.Type === 'Deleted')","\t\t\t\t{","\t\t\t\t\tvar tmpHasDeletedParameter = false;","","\t\t\t\t\t//first, check to see if filters are already looking for Deleted column","\t\t\t\t\tif (tmpFilter.length > 0)","\t\t\t\t\t{","\t\t\t\t\t\tfor (var x = 0; x < tmpFilter.length; x++)","\t\t\t\t\t\t{","\t\t\t\t\t\t\tif (tmpFilter[x].Column === tmpSchemaEntry.Column)","\t\t\t\t\t\t\t{","\t\t\t\t\t\t\t\ttmpHasDeletedParameter = true;","\t\t\t\t\t\t\t\tbreak;","\t\t\t\t\t\t\t}","\t\t\t\t\t\t}","\t\t\t\t\t}","\t\t\t\t\tif (!tmpHasDeletedParameter)","\t\t\t\t\t{","\t\t\t\t\t\t//if not, we need to add it","\t\t\t\t\t\ttmpFilter.push(","\t\t\t\t\t\t{","\t\t\t\t\t\t\tColumn: tmpTableName + '.' + tmpSchemaEntry.Column,","\t\t\t\t\t\t\tOperator: '=',","\t\t\t\t\t\t\tValue: 0,","\t\t\t\t\t\t\tConnector: 'AND',","\t\t\t\t\t\t\tParameter: 'Deleted'","\t\t\t\t\t\t});","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t}","","\t\tif (tmpFilter.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpWhere = ' WHERE';","","\t\t// This is used to disable the connectors for subsequent queries.","\t\t// Only the open parenthesis operator uses this, currently.","\t\tvar tmpLastOperatorNoConnector = false;","","\t\tfor (var i = 0; i < tmpFilter.length; i++)","\t\t{","\t\t\tif ((tmpFilter[i].Connector != 'NONE') && (tmpFilter[i].Operator != ')') && (tmpWhere != ' WHERE') && (tmpLastOperatorNoConnector == false))","\t\t\t{","\t\t\t\ttmpWhere += ' '+tmpFilter[i].Connector;","\t\t\t}","","\t\t\ttmpLastOperatorNoConnector = false;","","\t\t\tvar tmpColumnParameter;","","\t\t\tif (tmpFilter[i].Operator === '(')","\t\t\t{","\t\t\t\t// Open a logical grouping","\t\t\t\ttmpWhere += ' (';","\t\t\t\ttmpLastOperatorNoConnector = true;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === ')')","\t\t\t{","\t\t\t\t// Close a logical grouping","\t\t\t\ttmpWhere += ' )';","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IN')","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' ( :'+tmpColumnParameter+' )';","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IS NOT NULL')","\t\t\t{","\t\t\t\t// IS NOT NULL is a special operator which doesn't require a value, or parameter","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator;","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' :'+tmpColumnParameter;","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t}","","\t\treturn tmpWhere;","\t};","","\t/**","\t * Generate an ORDER BY clause from the sort array","\t *","\t * Each entry in the sort is an object like:","\t * {Column:'Color',Direction:'Descending'}","\t *","\t * @method: generateOrderBy","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateOrderBy = function(pParameters)","\t{","\t\tvar tmpOrderBy = pParameters.sort;","\t\tif (!Array.isArray(tmpOrderBy) || tmpOrderBy.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpOrderClause = ' ORDER BY';","\t\tfor (var i = 0; i < tmpOrderBy.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpOrderClause += ',';","\t\t\t}","\t\t\ttmpOrderClause += ' '+escapeColumn(tmpOrderBy[i].Column, pParameters);","","\t\t\tif (tmpOrderBy[i].Direction == 'Descending')","\t\t\t{","\t\t\t\ttmpOrderClause += ' DESC';","\t\t\t}","\t\t}","\t\treturn tmpOrderClause;","\t};","","\t/**","\t * Generate the limit clause","\t *","\t * @method: generateLimit","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateLimit = function(pParameters)","\t{","\t\tif (!pParameters.cap)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpLimit = ' LIMIT';","\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}","\t\t// Cap is required for a limit clause.","\t\ttmpLimit += ' ' + pParameters.cap;","","\t\treturn tmpLimit;","\t};","","\t/**","\t * Generate the update SET clause","\t *","\t * @method: generateUpdateSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateSetters = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpUpdate = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (pParameters.query.disableAutoDateStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateDate')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (pParameters.query.disableAutoUserStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateIDUser')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteDate':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// These are all ignored on update","\t\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tvar tmpColumnDefaultParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnDefaultParameter;","","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnDefaultParameter] = tmpRecords[0][tmpColumn];","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the update-delete SET clause","\t *","\t * @method: generateUpdateDeleteSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateDeleteSetters = function(pParameters)","\t{","\t\tif (pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t//Don't generate an UPDATE query if Delete tracking is disabled","\t\t\treturn false;","\t\t}","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCurrentColumn = 0;","\t\tvar tmpHasDeletedField = false;","\t\tvar tmpUpdate = '';","\t\t// No hash table yet, so, we will just linear search it for now.","\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\tvar tmpSchemaEntry = {Type:'Default'};","\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t{","\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\ttmpSchemaEntry = tmpSchema[i];","","\t\t\tvar tmpUpdateSql = null;","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'Deleted':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = 1';","\t\t\t\t\ttmpHasDeletedField = true; //this field is required in order for query to be built","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// Delete operation is an Update, so we should stamp the update time","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpSchemaEntry.Column+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\t//DON'T allow update of other fields in this query","\t\t\t\t\tcontinue;","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","","\t\t\ttmpUpdate += tmpUpdateSql;","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (!tmpHasDeletedField ||","\t\t\ttmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetValues = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpCreateSet += ',';","\t\t\t}","","\t\t\t//define a re-usable method for setting up field definitions in a default pattern","\t\t\tvar buildDefaultDefinition = function()","\t\t\t{","\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t// Set the query parameter","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpRecords[0][tmpColumn];","\t\t\t};","","\t\t\tvar tmpColumnParameter;","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NULL';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'AutoGUID':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse if (tmpRecords[0][tmpColumn] &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn].length >= 5 &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn] !== '0x0000000000000000') //stricture default","\t\t\t\t\t{","\t\t\t\t\t\t// Allow consumer to override AutoGUID","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.UUID;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\tif (pParameters.query.disableAutoDateStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NOW()';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\tif (pParameters.query.disableAutoUserStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use an appended number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpCreateSet === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpCreateSet;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetList = function(pParameters)","\t{","\t\t// The records were already validated by generateCreateSetValues","\t\tvar tmpRecords = pParameters.query.records;","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tdefault:","\t\t\t\t\tif (tmpCreateSet != '')","\t\t\t\t\t{","\t\t\t\t\t\ttmpCreateSet += ',';","\t\t\t\t\t}","\t\t\t\t\ttmpCreateSet += ' '+escapeColumn(tmpColumn, pParameters);","\t\t\t\t\tbreak;","\t\t\t}","\t\t}","","\t\treturn tmpCreateSet;","\t};","","","\tvar Create = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpCreateSetList = generateCreateSetList(pParameters);","\t\tvar tmpCreateSetValues = generateCreateSetValues(pParameters);","","\t\tif (!tmpCreateSetValues)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'INSERT INTO'+tmpTableName+' ('+tmpCreateSetList+') VALUES ('+tmpCreateSetValues+');';","\t};","","","\t/**","\t* Read one or many records","\t*","\t* Some examples:","\t* SELECT * FROM WIDGETS;","\t* SELECT * FROM WIDGETS LIMIT 0, 20;","\t* SELECT * FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20 WHERE LastName = 'Smith';","\t*","\t* @method Read","\t* @param {Object} pParameters SQL Query parameters","\t* @return {String} Returns the current Query for chaining.","\t*/","\tvar Read = function(pParameters)","\t{","\t\tvar tmpFieldList = generateFieldList(pParameters);","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpOrderBy = generateOrderBy(pParameters);","\t\tvar tmpLimit = generateLimit(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:tmpFieldList, TableName:tmpTableName, Where:tmpWhere, OrderBy:tmpOrderBy, Limit:tmpLimit});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Read Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT'+tmpFieldList+' FROM'+tmpTableName+tmpWhere+tmpOrderBy+tmpLimit+';';","\t};","","\tvar Update = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateSetters = generateUpdateSetters(pParameters);","","\t\tif (!tmpUpdateSetters)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateSetters+tmpWhere+';';","\t};","","\tvar Delete = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateDeleteSetters = generateUpdateDeleteSetters(pParameters);","","\t\tif (tmpUpdateDeleteSetters)","\t\t{","\t\t\t//If it has a deleted bit, update it instead of actually deleting the record","\t\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateDeleteSetters+tmpWhere+';';","\t\t}","\t\telse","\t\t{","\t\t\treturn 'DELETE FROM'+tmpTableName+tmpWhere+';';","\t\t}","\t};","","\tvar Count = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:[], TableName:tmpTableName, Where:tmpWhere, OrderBy:'', Limit:''});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Count Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT COUNT(*) AS RowCount FROM'+tmpTableName+tmpWhere+';';","\t};","","\tvar tmpDialect = ({","\t\tCreate: Create,","\t\tRead: Read,","\t\tUpdate: Update,","\t\tDelete: Delete,","\t\tCount: Count","\t});","","\t/**","\t * Dialect Name","\t *","\t * @property name","\t * @type string","\t */","\tObject.defineProperty(tmpDialect, 'name',","\t\t{","\t\t\tget: function() { return 'ALASQL'; },","\t\t\tenumerable: true","\t\t});","","\treturn tmpDialect;","};","","module.exports = new FoxHoundDialectALASQL();",""],"id":989},{"start":{"row":0,"column":0},"end":{"row":783,"column":0},"action":"insert","lines":["/**","* FoxHound ALASQL Dialect","*","* @license MIT","*","* For an ALASQL query override:","// An underscore template with the following values:","//      <%= DataElements %> = Field1, Field2, Field3, Field4","//      <%= Begin %>        = 0","//      <%= Cap %>          = 10","//      <%= Filter %>       = WHERE StartDate > :MyStartDate","//      <%= Sort %>         = ORDER BY Field1","// The values are empty strings if they aren't set.","*","* @author Steven Velozo <steven@velozo.com>","* @class FoxHoundDialectALASQL","*/","var libUnderscore = require('underscore');","","var FoxHoundDialectALASQL = function()","{","\t/**","\t * Generate a table name from the scope.","\t * ","\t * Because ALASQL is all in-memory, and can be run in two modes (anonymous","\t * working on arrays or table-based) we are going to make this a programmable","\t * value.  Then we can share the code across both providers.","\t *","\t * @method: generateTableName","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateTableName = function(pParameters)","\t{","\t\treturn ' '+pParameters.scope;","\t};","\t","\t/**","\t * Escape columns, because ALASQL has more reserved KWs than most SQL dialects","\t */","\tvar escapeColumn = (pColumn, pParameters) =>","\t{","\t\tif (pColumn.indexOf('.') < 0)","\t\t{","\t\t\treturn '`'+pColumn+'`';","\t\t}","\t\telse","\t\t{","\t\t\t// This could suck if the scope is not the same","\t\t\tvar tmpTableName = pParameters.scope;","\t\t\tif (pColumn.indexOf(tmpTableName+'.') > -1)","\t\t\t{","\t\t\t\treturn '`'+pColumn.replace(tmpTableName+'.', '')+'`';","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\t// This doesn't work well but we'll try it.","\t\t\t\treturn '`'+pColumn+'`';","\t\t\t}","\t\t}","\t};","","\t/**","\t * Generate a field list from the array of dataElements","\t *","\t * Each entry in the dataElements is a simple string","\t *","\t * @method: generateFieldList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateFieldList = function(pParameters)","\t{","\t\tvar tmpDataElements = pParameters.dataElements;","\t\tif (!Array.isArray(tmpDataElements) || tmpDataElements.length < 1)","\t\t{","\t\t\treturn ' *';","\t\t}","","\t\tvar tmpFieldList = ' ';","\t\tfor (var i = 0; i < tmpDataElements.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpFieldList += ', ';","\t\t\t}","\t\t\ttmpFieldList += escapeColumn(tmpDataElements[i], pParameters);","\t\t}","\t\treturn tmpFieldList;","\t};","","\t/**","\t * Generate a query from the array of where clauses","\t *","\t * Each clause is an object like:","\t\t{","\t\t\tColumn:'Name', ","\t\t\tOperator:'EQ', ","\t\t\tValue:'John', ","\t\t\tConnector:'And', ","\t\t\tParameter:'Name'","\t\t}","\t *","\t * @method: generateWhere","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the WHERE clause prefixed with WHERE, or an empty string if unnecessary","\t */","\tvar generateWhere = function(pParameters)","\t{","\t\tvar tmpFilter = Array.isArray(pParameters.filter) ? pParameters.filter : [];","\t\tvar tmpTableName = generateTableName(pParameters).trim();","","\t\tif (!pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t// Check if there is a Deleted column on the Schema. If so, we add this to the filters automatically (if not already present)","\t\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\tvar tmpSchemaEntry = tmpSchema[i];","","\t\t\t\tif (tmpSchemaEntry.Type === 'Deleted')","\t\t\t\t{","\t\t\t\t\tvar tmpHasDeletedParameter = false;","","\t\t\t\t\t//first, check to see if filters are already looking for Deleted column","\t\t\t\t\tif (tmpFilter.length > 0)","\t\t\t\t\t{","\t\t\t\t\t\tfor (var x = 0; x < tmpFilter.length; x++)","\t\t\t\t\t\t{","\t\t\t\t\t\t\tif (tmpFilter[x].Column === tmpSchemaEntry.Column)","\t\t\t\t\t\t\t{","\t\t\t\t\t\t\t\ttmpHasDeletedParameter = true;","\t\t\t\t\t\t\t\tbreak;","\t\t\t\t\t\t\t}","\t\t\t\t\t\t}","\t\t\t\t\t}","\t\t\t\t\tif (!tmpHasDeletedParameter)","\t\t\t\t\t{","\t\t\t\t\t\t//if not, we need to add it","\t\t\t\t\t\ttmpFilter.push(","\t\t\t\t\t\t{","\t\t\t\t\t\t\tColumn: tmpTableName + '.' + tmpSchemaEntry.Column,","\t\t\t\t\t\t\tOperator: '=',","\t\t\t\t\t\t\tValue: 0,","\t\t\t\t\t\t\tConnector: 'AND',","\t\t\t\t\t\t\tParameter: 'Deleted'","\t\t\t\t\t\t});","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t}","","\t\tif (tmpFilter.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpWhere = ' WHERE';","","\t\t// This is used to disable the connectors for subsequent queries.","\t\t// Only the open parenthesis operator uses this, currently.","\t\tvar tmpLastOperatorNoConnector = false;","","\t\tfor (var i = 0; i < tmpFilter.length; i++)","\t\t{","\t\t\tif ((tmpFilter[i].Connector != 'NONE') && (tmpFilter[i].Operator != ')') && (tmpWhere != ' WHERE') && (tmpLastOperatorNoConnector == false))","\t\t\t{","\t\t\t\ttmpWhere += ' '+tmpFilter[i].Connector;","\t\t\t}","","\t\t\ttmpLastOperatorNoConnector = false;","","\t\t\tvar tmpColumnParameter;","","\t\t\tif (tmpFilter[i].Operator === '(')","\t\t\t{","\t\t\t\t// Open a logical grouping","\t\t\t\ttmpWhere += ' (';","\t\t\t\ttmpLastOperatorNoConnector = true;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === ')')","\t\t\t{","\t\t\t\t// Close a logical grouping","\t\t\t\ttmpWhere += ' )';","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IN')","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' ( :'+tmpColumnParameter+' )';","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t\telse if (tmpFilter[i].Operator === 'IS NOT NULL')","\t\t\t{","\t\t\t\t// IS NOT NULL is a special operator which doesn't require a value, or parameter","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator;","\t\t\t}","\t\t\telse","\t\t\t{","\t\t\t\ttmpColumnParameter = tmpFilter[i].Parameter+'_w'+i;","\t\t\t\t// Add the column name, operator and parameter name to the list of where value parenthetical","\t\t\t\ttmpWhere += ' '+escapeColumn(tmpFilter[i].Column, pParameters)+' '+tmpFilter[i].Operator+' :'+tmpColumnParameter;","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpFilter[i].Value;","\t\t\t}","\t\t}","","\t\treturn tmpWhere;","\t};","","\t/**","\t * Generate an ORDER BY clause from the sort array","\t *","\t * Each entry in the sort is an object like:","\t * {Column:'Color',Direction:'Descending'}","\t *","\t * @method: generateOrderBy","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the field list clause","\t */","\tvar generateOrderBy = function(pParameters)","\t{","\t\tvar tmpOrderBy = pParameters.sort;","\t\tif (!Array.isArray(tmpOrderBy) || tmpOrderBy.length < 1)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpOrderClause = ' ORDER BY';","\t\tfor (var i = 0; i < tmpOrderBy.length; i++)","\t\t{","\t\t\tif (i > 0)","\t\t\t{","\t\t\t\ttmpOrderClause += ',';","\t\t\t}","\t\t\ttmpOrderClause += ' '+escapeColumn(tmpOrderBy[i].Column, pParameters);","","\t\t\tif (tmpOrderBy[i].Direction == 'Descending')","\t\t\t{","\t\t\t\ttmpOrderClause += ' DESC';","\t\t\t}","\t\t}","\t\treturn tmpOrderClause;","\t};","","\t/**","\t * Generate the limit clause","\t *","\t * @method: generateLimit","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateLimit = function(pParameters)","\t{","\t\tif (!pParameters.cap)","\t\t{","\t\t\treturn '';","\t\t}","","\t\tvar tmpLimit = ' LIMIT';","\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}","\t\t// Cap is required for a limit clause.","\t\ttmpLimit += ' ' + pParameters.cap;","","\t\treturn tmpLimit;","\t};","","\t/**","\t * Generate the update SET clause","\t *","\t * @method: generateUpdateSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateSetters = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpUpdate = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (pParameters.query.disableAutoDateStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateDate')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (pParameters.query.disableAutoUserStamp &&","\t\t\t\ttmpSchemaEntry.Type === 'UpdateIDUser')","\t\t\t{","\t\t\t\t// This is ignored if flag is set","\t\t\t\tcontinue;","\t\t\t}","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteDate':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// These are all ignored on update","\t\t\t\t\tcontinue;","\t\t\t}","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tvar tmpColumnDefaultParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdate += ' '+escapeColumn(tmpColumn, pParameters)+' = :'+tmpColumnDefaultParameter;","","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnDefaultParameter] = tmpRecords[0][tmpColumn];","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the update-delete SET clause","\t *","\t * @method: generateUpdateDeleteSetters","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateUpdateDeleteSetters = function(pParameters)","\t{","\t\tif (pParameters.query.disableDeleteTracking)","\t\t{","\t\t\t//Don't generate an UPDATE query if Delete tracking is disabled","\t\t\treturn false;","\t\t}","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCurrentColumn = 0;","\t\tvar tmpHasDeletedField = false;","\t\tvar tmpUpdate = '';","\t\t// No hash table yet, so, we will just linear search it for now.","\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\tvar tmpSchemaEntry = {Type:'Default'};","\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t{","\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\ttmpSchemaEntry = tmpSchema[i];","","\t\t\tvar tmpUpdateSql = null;","","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'Deleted':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = 1';","\t\t\t\t\ttmpHasDeletedField = true; //this field is required in order for query to be built","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\t\t// Delete operation is an Update, so we should stamp the update time","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = NOW()';","\t\t\t\t\tbreak;","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\tvar tmpColumnParameter = tmpSchemaEntry.Column+'_'+tmpCurrentColumn;","\t\t\t\t\ttmpUpdateSql = ' '+escapeColumn(tmpSchemaEntry.Column, pParameters)+' = :'+tmpColumnParameter;","\t\t\t\t\t// Set the query parameter","\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\t//DON'T allow update of other fields in this query","\t\t\t\t\tcontinue;","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpUpdate += ',';","\t\t\t}","","\t\t\ttmpUpdate += tmpUpdateSql;","","\t\t\t// We use a number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (!tmpHasDeletedField ||","\t\t\ttmpUpdate === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpUpdate;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetValues = function(pParameters)","\t{","\t\tvar tmpRecords = pParameters.query.records;","\t\t// We need to tell the query not to generate improperly if there are no values to set.","\t\tif (!Array.isArray(tmpRecords) || tmpRecords.length < 1)","\t\t{","\t\t\treturn false;","\t\t}","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tvar tmpCurrentColumn = 0;","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","","\t\t\tif (tmpCurrentColumn > 0)","\t\t\t{","\t\t\t\ttmpCreateSet += ',';","\t\t\t}","","\t\t\t//define a re-usable method for setting up field definitions in a default pattern","\t\t\tvar buildDefaultDefinition = function()","\t\t\t{","\t\t\t\tvar tmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t// Set the query parameter","\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = tmpRecords[0][tmpColumn];","\t\t\t};","","\t\t\tvar tmpColumnParameter;","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tcase 'AutoIdentity':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NULL';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'AutoGUID':","\t\t\t\t\tif (pParameters.query.disableAutoIdentity)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse if (tmpRecords[0][tmpColumn] &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn].length >= 5 &&","\t\t\t\t\t\t\ttmpRecords[0][tmpColumn] !== '0x0000000000000000') //stricture default","\t\t\t\t\t{","\t\t\t\t\t\t// Allow consumer to override AutoGUID","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.UUID;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateDate':","\t\t\t\tcase 'CreateDate':","\t\t\t\tcase 'DeleteDate':","\t\t\t\t\tif (pParameters.query.disableAutoDateStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is an autoidentity, so we don't parameterize it and just pass in NULL","\t\t\t\t\t\ttmpCreateSet += ' NOW()';","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tcase 'UpdateIDUser':","\t\t\t\tcase 'CreateIDUser':","\t\t\t\tcase 'DeleteIDUser':","\t\t\t\t\tif (pParameters.query.disableAutoUserStamp)","\t\t\t\t\t{","\t\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\t}","\t\t\t\t\telse","\t\t\t\t\t{","\t\t\t\t\t\t// This is the user ID, which we hope is in the query.","\t\t\t\t\t\t// This is how to deal with a normal column","\t\t\t\t\t\ttmpColumnParameter = tmpColumn+'_'+tmpCurrentColumn;","\t\t\t\t\t\ttmpCreateSet += ' :'+tmpColumnParameter;","\t\t\t\t\t\t// Set the query parameter","\t\t\t\t\t\tpParameters.query.parameters[tmpColumnParameter] = pParameters.query.IDUser;","\t\t\t\t\t}","\t\t\t\t\tbreak;","\t\t\t\tdefault:","\t\t\t\t\tbuildDefaultDefinition();","\t\t\t\t\tbreak;","\t\t\t}","","\t\t\t// We use an appended number to make sure parameters are unique.","\t\t\ttmpCurrentColumn++;","\t\t}","","\t\t// We need to tell the query not to generate improperly if there are no values set.","\t\tif (tmpCreateSet === '')","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn tmpCreateSet;","\t};","","\t/**","\t * Generate the create SET clause","\t *","\t * @method: generateCreateSetList","\t * @param: {Object} pParameters SQL Query Parameters","\t * @return: {String} Returns the table name clause","\t */","\tvar generateCreateSetList = function(pParameters)","\t{","\t\t// The records were already validated by generateCreateSetValues","\t\tvar tmpRecords = pParameters.query.records;","","\t\t// Check if there is a schema.  If so, we will use it to decide if these are parameterized or not.","\t\tvar tmpSchema = Array.isArray(pParameters.query.schema) ? pParameters.query.schema : [];","","\t\tvar tmpCreateSet = '';","\t\t// If there is more than one record in records, we are going to ignore them for now.","\t\tfor(var tmpColumn in tmpRecords[0])","\t\t{","\t\t\t// No hash table yet, so, we will just linear search it for now.","\t\t\t// This uses the schema to decide if we want to treat a column differently on insert","\t\t\tvar tmpSchemaEntry = {Column:tmpColumn, Type:'Default'};","\t\t\tfor (var i = 0; i < tmpSchema.length; i++)","\t\t\t{","\t\t\t\tif (tmpColumn == tmpSchema[i].Column)","\t\t\t\t{","\t\t\t\t\t// There is a schema entry for it.  Process it accordingly.","\t\t\t\t\ttmpSchemaEntry = tmpSchema[i];","\t\t\t\t\tbreak;","\t\t\t\t}","\t\t\t}","\t\t\tif (!pParameters.query.disableDeleteTracking)","\t\t\t{","\t\t\t\tif (tmpSchemaEntry.Type === 'DeleteDate' ||","\t\t\t\t\ttmpSchemaEntry.Type === 'DeleteIDUser')","\t\t\t\t{","\t\t\t\t\t// These are all ignored on insert (if delete tracking is enabled as normal)","\t\t\t\t\tcontinue;","\t\t\t\t}","\t\t\t}","\t\t\tswitch (tmpSchemaEntry.Type)","\t\t\t{","\t\t\t\tdefault:","\t\t\t\t\tif (tmpCreateSet != '')","\t\t\t\t\t{","\t\t\t\t\t\ttmpCreateSet += ',';","\t\t\t\t\t}","\t\t\t\t\ttmpCreateSet += ' '+escapeColumn(tmpColumn, pParameters);","\t\t\t\t\tbreak;","\t\t\t}","\t\t}","","\t\treturn tmpCreateSet;","\t};","","","\tvar Create = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpCreateSetList = generateCreateSetList(pParameters);","\t\tvar tmpCreateSetValues = generateCreateSetValues(pParameters);","","\t\tif (!tmpCreateSetValues)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'INSERT INTO'+tmpTableName+' ('+tmpCreateSetList+') VALUES ('+tmpCreateSetValues+');';","\t};","","","\t/**","\t* Read one or many records","\t*","\t* Some examples:","\t* SELECT * FROM WIDGETS;","\t* SELECT * FROM WIDGETS LIMIT 0, 20;","\t* SELECT * FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20;","\t* SELECT ID, Name, Cost FROM WIDGETS LIMIT 5, 20 WHERE LastName = 'Smith';","\t*","\t* @method Read","\t* @param {Object} pParameters SQL Query parameters","\t* @return {String} Returns the current Query for chaining.","\t*/","\tvar Read = function(pParameters)","\t{","\t\tvar tmpFieldList = generateFieldList(pParameters);","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpOrderBy = generateOrderBy(pParameters);","\t\tvar tmpLimit = generateLimit(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:tmpFieldList, TableName:tmpTableName, Where:tmpWhere, OrderBy:tmpOrderBy, Limit:tmpLimit});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Read Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT'+tmpFieldList+' FROM'+tmpTableName+tmpWhere+tmpOrderBy+tmpLimit+';';","\t};","","\tvar Update = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateSetters = generateUpdateSetters(pParameters);","","\t\tif (!tmpUpdateSetters)","\t\t{","\t\t\treturn false;","\t\t}","","\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateSetters+tmpWhere+';';","\t};","","\tvar Delete = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","\t\tvar tmpUpdateDeleteSetters = generateUpdateDeleteSetters(pParameters);","","\t\tif (tmpUpdateDeleteSetters)","\t\t{","\t\t\t//If it has a deleted bit, update it instead of actually deleting the record","\t\t\treturn 'UPDATE'+tmpTableName+' SET'+tmpUpdateDeleteSetters+tmpWhere+';';","\t\t}","\t\telse","\t\t{","\t\t\treturn 'DELETE FROM'+tmpTableName+tmpWhere+';';","\t\t}","\t};","","\tvar Count = function(pParameters)","\t{","\t\tvar tmpTableName = generateTableName(pParameters);","\t\tvar tmpWhere = generateWhere(pParameters);","","\t\tif (pParameters.queryOverride)","\t\t{","\t\t\ttry","\t\t\t{","\t\t\t\tvar tmpQueryTemplate = libUnderscore.template(pParameters.queryOverride);","\t\t\t\treturn tmpQueryTemplate({FieldList:[], TableName:tmpTableName, Where:tmpWhere, OrderBy:'', Limit:''});","\t\t\t}","\t\t\tcatch (pError)","\t\t\t{","\t\t\t\t// This pokemon is here to give us a convenient way of not throwing up totally if the query fails.","\t\t\t\tconsole.log('Error with custom Count Query ['+pParameters.queryOverride+']: '+pError);","\t\t\t\treturn false;","\t\t\t}","\t\t}","","\t\treturn 'SELECT COUNT(*) AS RowCount FROM'+tmpTableName+tmpWhere+';';","\t};","","\tvar tmpDialect = ({","\t\tCreate: Create,","\t\tRead: Read,","\t\tUpdate: Update,","\t\tDelete: Delete,","\t\tCount: Count","\t});","","\t/**","\t * Dialect Name","\t *","\t * @property name","\t * @type string","\t */","\tObject.defineProperty(tmpDialect, 'name',","\t\t{","\t\t\tget: function() { return 'ALASQL'; },","\t\t\tenumerable: true","\t\t});","","\treturn tmpDialect;","};","","module.exports = new FoxHoundDialectALASQL();",""]}],[{"start":{"row":261,"column":0},"end":{"row":266,"column":0},"action":"remove","lines":["\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}",""],"id":990}],[{"start":{"row":263,"column":0},"end":{"row":264,"column":0},"action":"insert","lines":["",""],"id":991}],[{"start":{"row":264,"column":0},"end":{"row":269,"column":0},"action":"insert","lines":["\t\t// If there is a begin record, we'll pass that in as well.","\t\tif (pParameters.begin !== false)","\t\t{","\t\t\ttmpLimit += ' ' + pParameters.begin + ',';","\t\t}",""],"id":992}],[{"start":{"row":268,"column":3},"end":{"row":269,"column":0},"action":"remove","lines":["",""],"id":993}],[{"start":{"row":269,"column":0},"end":{"row":270,"column":0},"action":"insert","lines":["",""],"id":994}],[{"start":{"row":267,"column":17},"end":{"row":267,"column":18},"action":"insert","lines":["F"],"id":995}],[{"start":{"row":267,"column":18},"end":{"row":267,"column":19},"action":"insert","lines":["E"],"id":996}],[{"start":{"row":267,"column":19},"end":{"row":267,"column":20},"action":"insert","lines":["T"],"id":997}],[{"start":{"row":267,"column":20},"end":{"row":267,"column":21},"action":"insert","lines":["C"],"id":998}],[{"start":{"row":267,"column":21},"end":{"row":267,"column":22},"action":"insert","lines":["H"],"id":999}],[{"start":{"row":267,"column":47},"end":{"row":267,"column":48},"action":"remove","lines":[","],"id":1000}],[{"start":{"row":267,"column":47},"end":{"row":267,"column":48},"action":"remove","lines":["'"],"id":1001}],[{"start":{"row":267,"column":46},"end":{"row":267,"column":47},"action":"remove","lines":["'"],"id":1002}],[{"start":{"row":267,"column":45},"end":{"row":267,"column":46},"action":"remove","lines":[" "],"id":1003}],[{"start":{"row":267,"column":44},"end":{"row":267,"column":45},"action":"remove","lines":["+"],"id":1004}],[{"start":{"row":267,"column":43},"end":{"row":267,"column":44},"action":"remove","lines":[" "],"id":1005}],[{"start":{"row":267,"column":22},"end":{"row":267,"column":23},"action":"insert","lines":[" "],"id":1006}]]},"ace":{"folds":[],"scrolltop":4107,"scrollleft":0,"selection":{"start":{"row":267,"column":23},"end":{"row":267,"column":23},"isBackwards":false},"options":{"guessTabSize":true,"useWrapMode":false,"wrapToView":true},"firstLineState":{"row":146,"state":"no_regex","mode":"ace/mode/javascript"}},"timestamp":1475069000000,"hash":"852144608675f5d659366762a3d363cdc5d763c2"}